var asdf = require("asdf-games");
// eslint-disable-next-line no-unused-vars
const { Container, TileSpriteXML, math, Texture, SpriteSheetXML } = asdf;

const texture = new Texture("../res/images/onlyObjects_default.png");
const xml = new SpriteSheetXML("../res/images/onlyObjects_default.xml");

class Bullet extends TileSpriteXML {
  constructor(player, type) {
    var name, speed;
    var colour = (player.remote) ? "Red" : "Blue";
    switch (type) {
    case 0: 
      name = `bullet${colour}1_outline.png`;
      speed = 5;
      break;
    case 1: 
      name = `bullet${colour}2_outline.png`;
      speed = 7.5;
      break;
    case 2: 
      name = `bullet${colour}3_outline.png`;
      speed = 10;
      break;
    }
    super(texture, xml, xml.findIndex("name", name));
    this.player = player;
    this.level = player.level;
    this.speed = speed;
  }

  update(dt, t) {
    super.update(dt, t);

    // Move
    this.pos.x += this.speed * Math.cos(this.rotation + 0.5 * Math.PI);
    this.pos.y += this.speed * Math.sin(this.rotation + 0.5 * Math.PI); 

    // Kill bullet when it leaves the level
    if (this.pos.x < 0 || this.pos.y < 0 || this.pos.x > this.level.w || this.pos.y > this.level.h) {
      this.dead = true;
    }
  }
}

class Tank extends TileSpriteXML {
  constructor(keys, mouse, level, colour, remote = false) {
    super(texture, xml, xml.findIndex("name", `tankBody_${colour.toLowerCase()}.png`));
    this.keys = keys;
    this.mouse = mouse;
    this.level = level;
    this.remote = remote;
    this.children = [];

    // Define body
    this.pos = { x: 250, y: 100 };
    this.pivot = { x: 17, y: 22 };
    this.speed = {
      move: 1
    };
    this.rotation = 0;

    this.locked = false;

    // Define barrel
    this.barrel = new TileSpriteXML(texture, xml, xml.findIndex("name", `tank${colour}_barrel2_outline.png`));
    this.barrel.pivot = { x: 6, y: 24 };
    this.barrel.pos = {
      x: 13,
      y: -3
    };
    this.children.push(this.barrel);

    this.firerate = {
      current: 0,
      min: 0.1,
      maxAmount: 5
    };

    // Define bullets
    this.bullets = new Container();
    this.level.children.push(this.bullets);
  }

  update(dt, t) {
    super.update(dt,t);
    
    if (!this.remote && this.locked == false) {
      // Make this.barrel follow the mouse
      var dx = (this.pos.x + this.barrel.pos.x) - this.mouse.pos.x;
      var dy = (this.pos.y + this.barrel.pos.y) - this.mouse.pos.y;
      var tan = dy / dx;
      var atan = Math.atan(tan);
      if (dy > 0 && dx > 0) {
        atan += Math.PI;
      }
      else if (dy < 0 && dx > 0) {
        atan -= Math.PI;
      }
      this.barrel.rotation = atan + (0.5 * Math.PI) - this.rotation;
    
      // Set rotation back to zero if higher than 2 * Pi (same angle)
      // TODO: Fix rotation, the barrel glitches a bit on 0 and 180 degreessa
      if (this.rotation >= (2 * Math.PI) || this.rotation <= -(2 * Math.PI)) {
        this.rotation = 0;
      }
      if (this.rotation < 0) {
        this.rotation = (2 * Math.PI) - this.rotation;
      }

      // TODO: If the angle is 90 or 270 degrees, allow the tank to move in that direction if 'a' or 's' is pressed
      // Rotate clockwise
      if (this.keys.x == 1) {
        this.rotation += dt * 2;
      }
      // Rotate anti-clockwise
      if (this.keys.x == -1) {
        this.rotation -= dt * 2;
      }

      // TODO: Make it easier to see what end of the tank is forward
      // Move
      if (this.keys.y == 1) {
        this.pos.x += this.speed.move * Math.cos(this.rotation + 0.5 * Math.PI);
        this.pos.y += this.speed.move * Math.sin(this.rotation + 0.5 * Math.PI);
      }
      if (this.keys.y == -1) {
        this.pos.x -= this.speed.move * Math.cos(this.rotation + 0.5 * Math.PI);
        this.pos.y -= this.speed.move * Math.sin(this.rotation + 0.5 * Math.PI);
      }

      // Shoot bullet on action key
      if (this.mouse.isDown && this.firerate.current >= this.firerate.min && this.bullets.children.length <= this.firerate.maxAmount) {
        var bullet = new Bullet(this, 0);
        bullet.rotation = this.barrel.rotation;
        bullet.pos = {
          x: this.pos.x / 1 + this.barrel.pivot.x + this.barrel.pos.x,
          y: this.pos.y / 1 + this.barrel.pivot.y + this.barrel.pos.y
        };
        this.bullets.add(bullet);
        this.firerate.current = 0;
      } else {
        if (this.firerate.current < this.firerate.min) {
          this.firerate.current += dt;
        }
      }
    }
  }
}

module.exports = Tank;