var io = require("socket.io-client");
var fs = require("fs");

const asdf = require("asdf-games");
// eslint-disable-next-line no-unused-vars
const { Container, Camera, Texture, SpriteSheetXML, TileMapXML, KeyControls, MouseControls } = asdf;

const Level = require("../helpers/Level.js");
const Tank = require("../entities/Tank.js");

class GameScreen extends Container {
  constructor(game, controls, onGameOver) {
    super();
    this.game = game;
    this.scene = this;
    this.controls = controls;
    this.onGameOver = onGameOver;
  
    this.controls.keys.reset();

    
    // Define tank colours
    const colours = ["Blue", "Red", "Green", "Sand"];

    // Keep local and external players 
    this.players = {};

    // Connect to TanksJS-Server instance
    this.socket = io("http://127.0.0.1:3000");
    this.socket.on("identify", () => {
      this.socket.emit("identification", {
        name: "Arn",
        playersMax: 2,
        gameID: "tank"
      });
      console.log(`Connected to server as ${this.socket.id}`);

      // Draw level
      var json = fs.readFileSync("./src/levels/default.json");
      this.level = new Level(JSON.parse(json), this.game, this.players);
      this.add(this.level);

      // Define local player
      this.players[this.socket.id] = new Tank(this.controls.keys, this.controls.mouse, this.level, colours[Object.keys(this.players).length]);
      this.player = this.players[this.socket.id];
      this.add(this.players[this.socket.id]);

      // Store current state of controls for comparing
      this.dataBuffer = JSON.stringify({
        pos: this.player.pos,
        rotation: {
          body: this.player.rotation,
          barrel: this.player.children[0].rotation
        }
      });

    });

    // Handle player logins    
    this.socket.on("roomUpdate", (data) => {
      // Add any player that is not the local player and is not already added
      for (const id in data) {
        if (id != this.socket.id && !(this.players[id])) {
          this.players[id] = new Tank(new KeyControls(false), new MouseControls(false), this.level, colours[Object.keys(this.players).length], true);
          this.add(this.players[id]);
        }
      }

      // Remove any player that disconnects from the game
      for (const id in this.players) {
        if (data[id] == undefined) {
          this.remove(this.players[id]);
          delete this.players[id];
        }
      }
    });

    // Handle player updates
    this.socket.on("update", (data) => {
      // TODO: Fix crashing related caused by this function
      const player = this.players[data.uuid];
      player.pos = data.player.pos;
      player.rotation = data.player.rotation.body;
      player.barrel.rotation = data.player.rotation.barrel;
    });

    // TODO: Lock player untill game starts
    this.socket.on("gameStart", () => {
      this.player.locked = false;
    });
  }


  
  update(dt, t) {
    super.update(dt, t); 

    if (this.player != undefined) {
      // Only send when controls are updated and minify data sent
      var data = {
        pos: this.player.pos,
        rotation: {
          body: this.player.rotation,
          barrel: this.player.children[0].rotation
        }
      };
      if (JSON.stringify(data) != this.dataBuffer) {
        console.log("Sending updates");
        this.socket.emit("update", data);
        this.dataBuffer = JSON.stringify(data);
      }
    }
  }
}

module.exports = GameScreen;